import { h, defineComponent, ref } from 'vue';
import { render } from '../../_utils';
export default defineComponent({
    name: 'TreeNodeContent',
    props: {
        clsPrefix: {
            type: String,
            required: true
        },
        disabled: {
            type: Boolean,
            default: false
        },
        onClick: Function,
        onDragstart: Function,
        tmNode: {
            type: Object,
            required: true
        }
    },
    setup(props) {
        const selfRef = ref(null);
        function doClick(e) {
            const { onClick } = props;
            if (onClick)
                onClick(e);
        }
        function handleClick(e) {
            doClick(e);
        }
        return {
            selfRef,
            handleClick
        };
    },
    render() {
        const { clsPrefix, handleClick, onDragstart, tmNode: { rawNode: { prefix, label, suffix } } } = this;
        return (h("span", { ref: "selfRef", class: [`${clsPrefix}-tree-node-content`], onClick: handleClick, draggable: onDragstart === undefined ? undefined : true, onDragstart: onDragstart },
            prefix ? (h("div", { class: `${clsPrefix}-tree-node-content__prefix` }, render(prefix))) : null,
            h("div", { class: `${clsPrefix}-tree-node-content__text` }, render(label)),
            suffix ? (h("div", { class: `${clsPrefix}-tree-node-content__suffix` }, render(suffix))) : null));
    }
});
